package be.iminds.ilabt.jfed.experimenter_gui.events;

import be.iminds.ilabt.jfed.espec.bundle.BundleFetcher;
import be.iminds.ilabt.jfed.espec.bundle.ESpecBundle;
import be.iminds.ilabt.jfed.experiment.ExperimentController;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Objects;

/**
 * User: twalcari
 * Date: 6/25/2015
 * Time: 17:43
 */
public class OpenESpecEvent {
    public enum ESpecSource { DIR, URL, ARCHIVE, GIT }

    @Nonnull private final ESpecSource eSpecSource;
    @Nullable private final File file;
    @Nullable private final URL url;
    @Nullable private final String gitSpec;

    @Nullable private final ESpecBundle directESpecBundle;

    // if not null, instead of a new experiment with the chosen ESpec,
    // the ESpec is loaded and started in the existing experiment
    @Nullable private final ExperimentController experimentController;

    public OpenESpecEvent(@Nullable ExperimentController experimentController, @Nonnull URL url) {
        this.experimentController = experimentController;
        this.file = null;
        this.eSpecSource = ESpecSource.URL;
        this.url = url;
        this.gitSpec = null;
        this.directESpecBundle = null;
    }

    public OpenESpecEvent(@Nullable ExperimentController experimentController, @Nonnull File file) {
        this.experimentController = experimentController;
        this.file = file;
        this.eSpecSource = file.isDirectory() ? ESpecSource.DIR : ESpecSource.ARCHIVE;
        this.url = null;
        this.gitSpec = null;
        this.directESpecBundle = null;
    }

    public OpenESpecEvent(@Nullable ExperimentController experimentController,
                          @Nonnull String any) throws MalformedURLException {
        this.experimentController = experimentController;
        this.directESpecBundle = null;
        if (any.startsWith("http")) {
            this.file = null;
            this.eSpecSource = ESpecSource.URL;
            this.url = new URL(any);
            this.gitSpec = null;
        } else {
            if (any.startsWith("git ")|| any.startsWith("github ")) {
                this.file = null;
                this.eSpecSource = ESpecSource.GIT;
                this.url = null;
                this.gitSpec = any;
            } else {
                this.file = new File(any);
                this.eSpecSource = file.isDirectory() ? ESpecSource.DIR : ESpecSource.ARCHIVE;
                this.url = null;
                this.gitSpec = null;
            }
        }
    }

    public OpenESpecEvent(@Nullable ExperimentController experimentController,
                          @Nonnull ESpecBundle eSpecBundle) throws MalformedURLException {
        this.experimentController = experimentController;
        this.file = null;
        this.eSpecSource = ESpecSource.ARCHIVE;
        this.url = null;
        this.gitSpec = null;
        this.directESpecBundle = eSpecBundle;
    }

    @Nonnull
    public ESpecBundle getEspecBundle(JFedPreferences jFedPreferences, GeniUserProvider geniUserProvider)
            throws InterruptedException, ESpecBundle.ESpecBundleInitException {
        if (directESpecBundle != null)
            return directESpecBundle;

        switch (eSpecSource) {
            case DIR: {
                if (file == null)
                    throw new NullPointerException("openESpecEvent.getFile() is not supposed to be null when eSpecSource="+eSpecSource);
                return BundleFetcher.fetchDir(file);
            }
            case ARCHIVE: {
                if (file == null)
                    throw new NullPointerException("openESpecEvent.getFile() is not supposed to be null when eSpecSource="+eSpecSource);
                return BundleFetcher.fetchArchiveFile(file);
            }
            case URL: {
                if (url == null)
                    throw new NullPointerException("openESpecEvent.getUrl() is not supposed to be null when eSpecSource="+eSpecSource);
                return BundleFetcher.fetchArchiveUrl(url);
            }
            case GIT: {
                if (gitSpec == null)
                    throw new NullPointerException("gitSpec is not supposed to be null when eSpecSource="+eSpecSource);
                return BundleFetcher.fetchGitRepoDir(
                        gitSpec,
                        jFedPreferences.getGitAuthPreferences(geniUserProvider.getLoggedInGeniUser())
                );
            }
            default: throw new RuntimeException("ESpecSource "+eSpecSource+" is not supported");
        }
    }

    @Nullable
    public ExperimentController getExperimentController() {
        return experimentController;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof OpenESpecEvent)) return false;
        OpenESpecEvent that = (OpenESpecEvent) o;
        return eSpecSource == that.eSpecSource &&
                Objects.equals(file, that.file) &&
                Objects.equals(url, that.url) &&
                Objects.equals(gitSpec, that.gitSpec);
    }

    @Override
    public int hashCode() {

        return Objects.hash(eSpecSource, file, url, gitSpec);
    }

    @Override
    public String toString() {
        return "OpenESpecEvent{" +
                "eSpecSource=" + eSpecSource +
                ", file=" + file +
                ", url=" + url +
                ", gitSpec='" + gitSpec + '\'' +
                '}';
    }
}
